## Rekit

Rekit is a toolkit for building scalable web applications with React, Redux and React-router.

It helps you focus on application logics instead of dealing with massive libraries, patterns, configurations etc.

A Rekit application is extremely scalable, testable and maintainable by using feature-oriented folder structure, one action per file pattern. This ensures application elements are well decoupled.

With Rekit sublime text plugin, it's super easy to create features, components, actions, tests, etc. And you can even test or build your applications without leaving the editor. Besides the plugin, all the features are also available as npm scripts, they could be used within a terminal.

Here is a quick look:

## Installation

## Sublimetext Plugin

## Motivation
Creating web applications is hard today because there are too many technologies to learn. But it's also charming today, because there are so many great libraries helping to build complicated things in elegant ways. After enjoying creating a large application mainly with React, Redux and React-router, I decided to summarize it and make the approach re-usable, so I created Rekit.

So this is a very opinionate approach of building web applications. And the approach is guranteed by tools. Almost all project elements could be generated by provided command line tools (also available as Rekit sublime plugin). So folder structure, naming, configurations could be always consistent so that they are easy to understand, analyze, maintain and test.

## Features

## Philosophy

## About docs
Since 






Besides creating empty projects with configurations ready, it also provides scripts to generate code for frequently used elements such as components, actions, reducers etc. And it provides the ability to delete them.

The Rekit project boilerplate is designed to be extensible and production-ready, so all technical artifacts are grouped by features. There is a folder for each feature and all feature related actions, reducers, components and pages are put under the folder. Such folder structure supports large and complicated projects.

See the Online Example
=====
There is a simple forum application created with Rekit for reference: [http://github.com/supnate/rekit-example](http://github.com/supnate/rekit-example)

Install
======
```
npm install -g rekit
```
This will install a global command `rekit` to the system. Run the command to create a new project.

A Quick Look
======
When your project is created and npm installed. You can use npm scripts in your project, for example, by running:
```
npm run add:async-action my-feature/save-article
```
You will get code generated for the Redux async action as below:
![Async action code.](/images/async_action.png?raw=true)

More npm scripts usage, see the below documentation.

What Rekit Does
======
1. Create a React + Redux + React-router based project boilerplate with all necessary configurations.
2. Provide command line tools in your project to create and delete features, actions, reducers, components and pages.

More Than Boilerplate
======
Rekit uses a fixed project folder structure and follows strict naming rules, so it could do more things besides only generating code. For examples, it auto register a route path when creating a page component. It auto binds reducers when creating a new feature. The opposite actions are taken when deleting such elements. You can look at below command line tools doc for more details.

Motivation
======
I've been using the draft version of this toolkit for about half of a year and found it is really helpful for improve development efficiency and it also helps to make code consistent among different application logic. Now I'm about to give an online session about how to use React and Redux, so I created this project and the example project to better introduce the technical details. The project was initially created as [react-init](https://github.com/supnate/react-init) but that name has existed on npm so I moved it here and named it Rekit.

Key Features
======
 * Always pull the latest package versions when creating a project. That is the new created project always has the latest dependencies.
 * All command line tools are copied to the project so that you fully own it, you can modify the tools to better match your project needs. Once the project is created, Rekit itself is no longer needed.
 * The project structure is designed to be flexible, extensible and predictable.
 * It's production-ready but not a starter kit.
 * Embed build script and test server for the build result.
 * Use [Webpack](http://webpack.github.io) for bundling.
 * Use [Babel](https://babeljs.io/) for ES2015(ES6)+ support.
 * Use [React hot loader](http://gaearon.github.io/react-hot-loader/) for hot module replacement.
 * Use [Redux](http://redux.js.org/) for application state management.
 * Use immutable state management.
 * Use [React-router](https://github.com/reactjs/react-router) for routing and it's configured with Redux reducer.
 * 404 page configured for React-router.
 * Use [Webpack dll plugin](http://webpack.github.io/docs/list-of-plugins.html#dllplugin) to improve dev-time build performance.
 * Use [Less](http://lesscss.org/) as CSS transpiler.
 * Use [eslint-config-airbnb](https://github.com/airbnb/javascript) for code style check.
 * Support [Redux dev tools](https://chrome.google.com/webstore/detail/redux-devtools/lmhkpmbekcpmknklioeibfkpmmfibljd).
 * [Webpack Dashboard](https://github.com/FormidableLabs/webpack-dashboard) integrated.
 * Command line tools to mange actions, reducers, components and pages.

Design Philosophy
======
1. ***Different application logic for a single functionality should be placed in one place.***
You don't want to jump among different folders to edit code for the same functionality. Such as one functionality usually contains component, actions, reducers, styes, etc. Navigating between these files takes too much time when the project grows large and becomes complicated. So Rekit organizes different parts of a feature into one folder. You can finish a functionality just in one folder.

2. ***Developer should own every line of the project configuration.***
Configuration errors often cause weird behaviors of your application and they are usually very hard to debug. If you don't know how each part of your project works together, you are at the risk of wasting tons of time on configuration errors. So Rekit just creates the project with an initial correct config and put every line of configuration into your project, you should understand and is responsible to maintain it.

3. ***Developer should understand what a tool exactly does when using it.***
Tools could accelerate your development and reduce errors. But if you don't understand what it does behind, then you are at risk. So Rekit copies all tools with source code into your project instead of hiding it behind some commands. The source code is just there for your reference.

4. ***Developer should have his own tool set for the development.***
Rekit provides a basic tool set for a typical React + Redux project. Once the project is created, the tool set has no relationship with Rekit. So they could be easily customized so that it better fits the specific project needs.

Project Folder structure
=====
A Rekit project has below folder structure:

```
|-- project-name
|    |-- src
|    |    |-- features
|    |    |    |-- home
|    |    |    |    |-- actions.js
|    |    |    |    |-- constants.js
|    |    |    |    |-- index.js
|    |    |    |    |-- reducer.js
|    |    |    |    |-- route.js
|    |    |    |    |-- styles.less
|    |    |    |    |-- ...
|    |    |    +-- feature-1
|    |    |    +-- feature-2
|    |    |-- components
|    |    |-- common
|    |    |-- containers
|    |    +-- styles
|    |-- tools
|    |    +-- feature_template
|    |    |-- server.js
|    |    |-- build.js
|    |    |-- ...
|-- .eslintrc
|-- .gitignore
|-- .webpack.dev.config.js
|-- ...
```

This Rekit repo itself is also a Rekit project, you can also browse the project source code to understand the folder structure. Actually this repo itself is the base for creating a new project.

Concepts
======
Before starting a Rekit project, some basic concepts of the project need to be explained:
### Feature
This is the top level concept of a project, also it may be the only new concept here. Each feature corresponds to a logical part of an application. For example, an EShop application usually contains below features:
 * `customer`: manage basic customer information.
 * `product`: manage products on sale.
 * `category`: manage product categories.
 * `order`: manage sales orders.
 * `user`: manage admins of the system.
 * etc...

A feature usually always contains multiple actions, components and pages.

### Component
It's just React component. In a Rekit project, there are two types of component: one is common component which is not related with any features, it's put in the `src/components` folder. The other is feature component which is provided by a feature, so it is put under the feature folder.

### Page
Page is some of special component which maps to the concept of 'Container' in the best practice of [separating presentational and container components](http://redux.js.org/docs/basics/UsageWithReact.html) pattern. A page also usually maps to a specific URL path, so when creating a page also needs to create a routing definition.

### Action
It's just [Redux action](http://redux.js.org/docs/basics/Actions.html).

### Async action
When developing a web application, we often need to request data from server side. It needs to be implemented as an `async-action`. Actually it is not anything new but just combining some normal actions and reducers. In Rekit, when creating an async-action, it creates the code skeleton to handle request begin, request pending, request success, request failure action types. And maintain the requestPending, requestError state in the reducer. With the command line tool `npm run add:async-action` it automatically creates the skeleton and you only need to fill the application logic in different technical artifacts. See below command line tools guide to see more details.

### Reducer
It's just [Redux reducer](http://redux.js.org/docs/basics/Reducers.html).

### Container
The top level container of the application, it usually defines the top level UI layout and it's the container of pages. They are put in `src/containers` folder. Very few containers are needed.

